// AccelerometerVisualization for the Zune HD by Charles Petzold, April 2010
using System;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace AccelerometerVisualization
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        const int BALL_RADIUS = 8;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Viewport viewport;
        SpriteFont kootenay14;
        float textHeight;

        Vector3 minAcceleration = Vector3.One;
        Vector3 maxAcceleration = Vector3.Zero;

        int unitRadius;
        Vector2 screenCenter;
        Texture2D backgroundTexture;
        Vector2 backgroundTextureCenter;
        Texture2D ballTexture;
        Vector2 ballTextureCenter;
        Vector2 ballPosition;
        float ballScale;
        bool isZNegative;

        StringBuilder stringBuilder = new StringBuilder();

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromSeconds(1 / 30.0);
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(this.GraphicsDevice);
            viewport = this.GraphicsDevice.Viewport;
            kootenay14 = this.Content.Load<SpriteFont>("Kootenay14");
            textHeight = kootenay14.MeasureString(" ").Y;

            screenCenter = new Vector2(viewport.Width / 2, viewport.Height / 2);
            int smallerDimension = Math.Min(viewport.Width, viewport.Height);
            unitRadius = (smallerDimension - BALL_RADIUS) / 2;

            // Create and draw background texture
            backgroundTexture = new Texture2D(this.GraphicsDevice, smallerDimension, smallerDimension);
            backgroundTextureCenter = new Vector2(smallerDimension / 2, smallerDimension / 2);

            Color[] pixels = new Color[backgroundTexture.Width * backgroundTexture.Height];

            // Draw horizontal line
            for (int x = 0; x < backgroundTexture.Width; x++)
                SetPixel(backgroundTexture, pixels, x, backgroundTexture.Height / 2, Color.White);

            // Draw vertical line
            for (int y = 0; y < backgroundTexture.Height; y++)
                SetPixel(backgroundTexture, pixels, backgroundTexture.Width / 2, y, Color.White);

            DrawCenteredCircle(backgroundTexture, pixels, unitRadius);
            DrawCenteredCircle(backgroundTexture, pixels, unitRadius / 2);
            backgroundTexture.SetData<Color>(pixels);

            // Create and draw ball texture
            ballTexture = new Texture2D(this.GraphicsDevice, 2 * BALL_RADIUS, 2 * BALL_RADIUS);
            ballTextureCenter = new Vector2(BALL_RADIUS, BALL_RADIUS);
            pixels = new Color[ballTexture.Width * ballTexture.Height];
            DrawFilledCenteredCircle(ballTexture, pixels, BALL_RADIUS);
            ballTexture.SetData<Color>(pixels);
        }

        void DrawCenteredCircle(Texture2D texture, Color[] pixels, int radius)
        {
            Point center = new Point(texture.Width / 2, texture.Height / 2);
            int halfPoint = (int)(0.707 * radius);

            for (int y = -halfPoint; y <= halfPoint; y++)
            {
                int x1 = (int)Math.Round(Math.Sqrt(radius * radius - Math.Pow(y, 2)));
                int x2 = -x1;

                SetPixel(texture, pixels, x1 + center.X, y + center.Y, Color.White);
                SetPixel(texture, pixels, x2 + center.X, y + center.Y, Color.White);

                // Since symmetric, just swap coordinates for other piece
                SetPixel(texture, pixels, y + center.X, x1 + center.Y, Color.White);
                SetPixel(texture, pixels, y + center.X, x2 + center.Y, Color.White);
            }
        }

        void DrawFilledCenteredCircle(Texture2D texture, Color[] pixels, int radius)
        {
            Point center = new Point(texture.Width / 2, texture.Height / 2);

            for (int y = -radius; y < radius; y++)
            {
                int x1 = (int)Math.Round(Math.Sqrt(radius * radius - Math.Pow(y, 2)));

                for (int x = -x1; x < x1; x++)
                    SetPixel(texture, pixels, x + center.X, y + center.Y, Color.White);
            }
        }

        void SetPixel(Texture2D texture, Color[] pixels, int x, int y, Color clr)
        {
            pixels[y * texture.Width + x] = clr;
        }

        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            Vector3 acceleration = Accelerometer.GetState().Acceleration;

            if (acceleration.LengthSquared() > maxAcceleration.LengthSquared())
                maxAcceleration = acceleration;

            if (acceleration.LengthSquared() < minAcceleration.LengthSquared())
                minAcceleration = acceleration;

            stringBuilder.Remove(0, stringBuilder.Length);
            stringBuilder.AppendFormat("Now: ({0:F2}, {1:F2}, {2:F2}) = {3:F2}\n", acceleration.X, acceleration.Y, acceleration.Z, acceleration.Length());
            stringBuilder.AppendFormat("Min: ({0:F2}, {1:F2}, {2:F2}) = {3:F2}\n", minAcceleration.X, minAcceleration.Y, minAcceleration.Z, minAcceleration.Length());
            stringBuilder.AppendFormat("Max: ({0:F2}, {1:F2}, {2:F2}) = {3:F2}", maxAcceleration.X, maxAcceleration.Y, maxAcceleration.Z, maxAcceleration.Length());

            ballScale = acceleration.Length();
            ballPosition = new Vector2(screenCenter.X + unitRadius * acceleration.X / ballScale,
                                       screenCenter.Y - unitRadius * acceleration.Y / ballScale);
            isZNegative = acceleration.Z < 0;

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Navy);

            spriteBatch.Begin();
            spriteBatch.Draw(backgroundTexture, screenCenter, null, Color.White, 0, backgroundTextureCenter, 1, SpriteEffects.None, 0);
            spriteBatch.Draw(ballTexture, ballPosition, null, isZNegative ? Color.Red : Color.Lime, 0, ballTextureCenter, ballScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(kootenay14, stringBuilder, Vector2.Zero, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
